var express = require('express');
var cors = require('cors');
var router = express.Router();

// firebase Admin 써드파티 미들웨어
const admin = require('firebase-admin');

// firebase Admin 초기화
const firebaseAdmin = admin.initializeApp({
  credential: admin.credential.cert({
    // firebase admin sdk json 에 있는 키값들을 넣어준다.
  }),
  databaseURL: 
  storageBucket: 
});

// firebase firestore 선언
let db = firebaseAdmin.firestore();

/* GET home page. 테스트용 코드 */
router.get('/', function(req, res, next) {
  res.render('index', { title: 'Express' });
});

router.get('/chat', (req, res, next) => {
  console.log(req.query.test);
  res.send("chatget send");
});

router.all('/chatall', (req, res, next) => {
  console.log(req.body.test);

  res.send("chatall send!");
  //res.redirect('/');
});

router.post('/chatpost', (req, res, next) => {
  console.log(req.body.test);

  res.send("chatpost send!");
  //res.redirect('/');
});


// 신규 유저 생성 후 DB 세팅 및 저장
router.post('/newUserCreate', (req, res, next) => {
  const work = async () => {
    var user = req.body;

    // uid 가 없다면 오류 처리
    if (!user.uid)
    {
      res.send(false);
      return;
    }

    // cloud Firestore 에 새로운 유저 데이터 등록
    db.collection('userData').doc(user.uid).set(user)
    .then(() => {
    // 성공시
    console.log(user.uid + " 정보 생성완료!");
    return res.send(true);
    }, (err) => {
    // 실패시
    console.log(err);

    return res.send(false);
    });
  }
  
  work().catch((err) => {
    console.log(err);
    return res.send(err);
  });
});

// 신규 유저 인벤토리 DB 세팅 및 저장
router.post('/newUserInvenCreate', (req, res, next) => {
  const work = async () => {
    var user = req.body;

    // uid 가 없다면 오류 처리
    if (!user.uid)
    {
      return res.send(false);
    }
    
    // cloud Firestore 에 새로운 유저 인벤토리 정보를 저장한다.
    db.collection('userInventory').doc(user.uid).set(user)
    .then(() => {
      // 성공시
      console.log(user.uid + " 인벤토리 생성완료!");

      return res.send(true);
    }, (err) => {
      // 실패시
      console.log(err);

      return res.send(false);
    });
  }

  work().catch((err) => {
    console.log(err);
    return res.send(err);
  });
});

// 영수증 정보 저장
router.post('/userReceiptSave', (req, res) => {
  const work = async () => {
    try
    {
      var receiptData = req.body;

      // 저장
      // 중복의 영수증값이 있다면 덮어쓴다. (자동으로 덮어쓴다.)
      db.collection('receipt').doc(receiptData.transaction_id).set(receiptData)
      .then(() => {
        console.log(req.body.transaction_id + " 결재 영수증 저장");
        return res.send(true);
      }, (err) => {
        console.log(err);
        return res.send(false);
      });
    } catch(err)
    {
      console.log(err);

      return res.send(false);
    }
  }
  
  work();
});

// 영수증 정보 갱신
router.post('/userReceiptUpdate', (req, res) => {
  const work = async () => {
    try
    {
      console.log(req.body.tid + " 갱신 시작");

      // 특정 영수증의 transaction_id 를 기준으로 DB에서 검색 후 givecheck 를 0 -> 1로 수정한다.
      doc = firebaseAdmin.firestore().collection('receipt').doc(req.body.tid).update({
        givecheck: "1"
      })
      .then(() => {
        return res.send(true);
      })
      .catch((err) => 
      {
        console.log(err);
        return res.send(false);
      });
    }
    catch(err)
    {
      console.log(err);
      return res.send(false);
    }
  };
  
  work();
});

// 유저 코인 정보를 갱신
router.post('/userRenewCoin', (req, res) => {
  const work = async () => {
    console.log(req.body.uid + " 코인 지급 완료");

    var coinData = req.body;

    coin = Number(coinData.coin);
    // uid로 검색한 유저의 코인을 firebase 에서 지원하는 firestore.FieldValue.increment 함수를 이용하여
    // 클라에서 보낸 코인 수 만큼 늘려준다.
    db.collection('userInventory').doc(coinData.uid).update({
      coin: admin.firestore.FieldValue.increment(coin)
    }).then(() => {
      // 성공시
      console.log(coinData.uid + " 의 유저의 코인 증가 : " + coinData.coin);
      return res.send(true);
    }).catch((err) => {
      console.log(err);
      return res.send(false);
    });
  };
  
  work();
});

// 특정 유저 데이터를 json으로 리턴해준다.
router.get('/userDataReturn', (req, res) => {
  const work = async () => {
    try
    {
      console.log(req.query.uid + " 유저 데이터 조회");

      var uid = req.query.uid;

      // uid를 이용하여 유저 정보를 db에서 검색한다.
      db.collection('userData').doc(uid).get()
      .then((snapshot) => {
        var userData = snapshot.data();

        if (!userData.uid)
        {
          return res.send(uid + " 유저 데이터 조회 실패");
        }

        return res.json(userData);
      }).catch((err) => {
        console.log(err);

        return res.send(err);
      });
    }
    catch(err)
    {
      console.log(err);
      
      return res.send(err);
    }
  };

  work();
});

// 특정 유저 인벤 정보를 json으로 리턴해준다.
router.get('/userInvenDataReturn', (req, res) => {
  const work = async () => {
    try
    {
      console.log(req.query.uid + " 유저 인벤 데이터 조회");

      var uid = req.query.uid;

      db.collection('userInventory').doc(uid).get()
      .then((snapshot) => {
        var userData = snapshot.data();

        if (!userData.uid)
        {
          return res.send(uid + " 유저 데이터 조회 실패");
        }

        return res.json(userData);
      }).catch((err) => {
        console.log(err);

        return res.send(err);
      });
    }
    catch(err)
    {
      console.log(err);
      return res.send(err);
    }
  };

  work();
});

// 테스트용 DB 삭제
 router.post('/testinit', (req, res) => {
   console.log("Test Init Start...");

   try
   {
     db.collection('userData').doc(req.body.uid).delete();
   }
   catch(err)
   {
     console.log(err);
     res.send(false);
   }

   console.log(req.body.uid + " 삭제 완료");
   res.send(true);   
 });

module.exports = router;
